@@ -1,6 +1,6 @@
 package Math::Polynomial::Solve;
 
-require 5.006;
+require 5.008003;
 
 use Math::Complex;
 use Carp;
@@ -8,7 +8,12 @@ use Exporter;
 use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 use strict;
 use warnings;
-#use Smart::Comments;
+
+#
+# Three # for "I am here" messages, four # for variable dumps.
+# Five # for a dump of the companion matrix.
+#
+#use Smart::Comments q(####);
 
 @ISA = qw(Exporter);
 
@@ -42,12 +47,15 @@ use warnings;
 		epsilon
 		fltcmp
 		poly_iteration
+		poly_tolerance
+		newtonraphson
 		laguerre
 		poly_antiderivative
 		poly_derivative
 		poly_constmult
 		poly_divide
 		poly_evaluate
+		poly_derivaluate
 		poly_nonzero_term_count
 		simplified_form
 	) ],
@@ -56,7 +64,7 @@ use warnings;
 @EXPORT_OK = ( @{ $EXPORT_TAGS{'classical'} }, @{ $EXPORT_TAGS{'numeric'} },
 	@{ $EXPORT_TAGS{'sturm'} }, @{ $EXPORT_TAGS{'utility'} } );
 
-our $VERSION = '2.61';
+our $VERSION = '2.63';
 
 #
 # Options to set or unset to force poly_roots() to use different
@@ -87,6 +95,7 @@ my %option = (
 #
 my %iteration = (
 	hessenberg => 60,
+	newtonraphson => 60,
 	laguerre => 60,
 	sturm_bisection => 100,
 );
@@ -96,6 +105,7 @@ my %iteration = (
 # replaced in the BEGIN block.
 #
 my %tolerance = (
+	newtonraphson => 1e-14,
 	laguerre => 1e-14,
 	fltcmp => 1.5e-8,
 );
@@ -119,6 +129,7 @@ BEGIN
 	}
 
 	$tolerance{laguerre} = 2 * $epsilon;
+	$tolerance{newtonraphson} = 2 * $epsilon;
 }
 
 #
@@ -127,9 +138,8 @@ BEGIN
 #
 sub sign
 {
-	my($x) = @_;
-	return 1 if ($x > 0);
-	return -1 if ($x < 0);
+	return 1 if ($_[0] > 0);
+	return -1 if ($_[0] < 0);
 	return 0;
 }
 
@@ -144,7 +154,6 @@ sub sign
 sub fltcmp
 {
 	my($a, $b) = @_;
-	#my($flt) = 0.25/16777216;	# a good enough value for testing.
 
 	return -1 if ($a + $tolerance{fltcmp} < $b);
 	return 1 if ($a - $tolerance{fltcmp} > $b);
@@ -172,11 +181,13 @@ sub epsilon
 #
 sub get_hessenberg
 {
+	carp "get_hessenberg() is DEPRECATED. Please use \"\%opts = poly_option(); \$opts{hessenberg};\" instead.";
 	return $option{hessenberg};
 }
 
 sub set_hessenberg
 {
+	carp "set_hessenberg() is DEPRECATED. Please use \"poly_option(hessenberg => $_[0]);\" instead.";
 	$option{hessenberg} = ($_[0])? 1: 0;
 }
 
@@ -513,9 +524,10 @@ sub quartic_roots
 	return ($x[0] - $b4, $x[1] - $b4, $x[2] - $b4, $x[3] - $b4);
 }
 
-
+#
 # Perl code to find roots of a polynomial translated by Nick Ing-Simmons
-# <Nick@Ing-Simmons.net> from FORTRAN code by Hiroshi Murakami.
+# from FORTRAN code by Hiroshi Murakami.
+#
 # From the netlib archive: http://netlib.bell-labs.com/netlib/search.html
 # In particular http://netlib.bell-labs.com/netlib/opt/companion.tgz
 
@@ -586,12 +598,12 @@ sub build_companion
 	}
 
 	#
-	### Now we balance the matrix.
-	#### @h
+	##### @h
 	#
-	# Balancing the unsymmetric matrix A.
-	# Perl code translated by Nick Ing-Simmons <Nick@Ing-Simmons.net>
-	# from FORTRAN code by Hiroshi Murakami.
+	### Balancing the unsymmetric matrix A.
+	#
+	# Perl code translated by Nick Ing-Simmons from FORTRAN code
+	# by Hiroshi Murakami.
 	#
 	#  The Fortran code is based on the Algol code "balance" from paper:
 	#  "Balancing a Matrix for Calculation of Eigenvalues and Eigenvectors"
@@ -708,19 +720,19 @@ sub hqr_eigen_hessenberg
 	my $n   = $#h;
 
 	#
+	# Eigenvalue Computation by the Householder QR method for the
+	# Real Hessenberg matrix.
+	#
+	# Perl code translated by Nick Ing-Simmons from FORTRAN code
+	# by Hiroshi Murakami.
 	#
-	# Eigenvalue Computation by the Householder QR method for the Real Hessenberg matrix.
-	# Perl code translated by Nick Ing-Simmons <Nick@Ing-Simmons.net>
-	# from FORTRAN code by Hiroshi Murakami.
 	# The Fortran code is based on the Algol code "hqr" from the paper:
 	#   "The QR Algorithm for Real Hessenberg Matrices"
 	#   by R. S. Martin, G. Peters and J. H. Wilkinson,
 	#   Numer. Math. 14, 219-231(1970).
 	#
-	#
 	my($p, $q, $r);
 	my($w, $x, $y);
-	my($s, $z );
 	my $t = 0.0;
 
 	my @w;
@@ -736,11 +748,16 @@ sub hqr_eigen_hessenberg
 			#
 			# Look for single small sub-diagonal element;
 			#
-			my $l;
-			for ($l = $n; $l >= 2; $l--)
+			my $l = 1;
+			for my $d (reverse 2 .. $n)
 			{
-				last if ( abs( $h[$l][ $l - 1 ] ) <= $epsilon *
-					( abs( $h[ $l - 1 ][ $l - 1 ] ) + abs( $h[$l][$l] ) ) );
+				if (abs( $h[$d][ $d - 1 ] ) <= $epsilon *
+				    (abs( $h[ $d - 1 ][ $d - 1 ] ) +
+				     abs( $h[$d][$d] ) ) )
+				{
+					$l = $d;
+					last;
+				}
 			}
 
 			$x = $h[$n][$n];
@@ -804,7 +821,7 @@ sub hqr_eigen_hessenberg
 					$h[$i][$i] -= $x;
 				}
 
-				$s = abs($h[$n][$na]) + abs($h[$na][$n - 2]);
+				my $s = abs($h[$n][$na]) + abs($h[$na][$n - 2]);
 				$y = 0.75 * $s;
 				$x = $y;
 				$w = -0.4375 * $s * $s;
@@ -813,34 +830,49 @@ sub hqr_eigen_hessenberg
 			$its++;
 
 			#
-			# Look for two consecutive small sub-diagonal elements.
+			### Look for two consecutive small
+			### sub-diagonal elements.
 			#
-			my $m;
-			for ($m = $n - 2 ; $m >= $l ; $m--)
+			my $m = $l;	# Set in case we fall through the loop.
+			for my $d (reverse $l .. $n - 2)
 			{
-				$z = $h[$m][$m];
+				my $z = $h[$d][$d];
+				my $s = $y - $z;
 				$r = $x - $z;
-				$s = $y - $z;
-				$p = ($r * $s - $w) / $h[$m + 1][$m] + $h[$m][$m + 1];
-				$q = $h[$m + 1][$m + 1] - $z - $r - $s;
-				$r = $h[$m + 2][$m + 1];
+				$p = ($r * $s - $w) / $h[$d + 1][$d] + $h[$d][$d + 1];
+				$q = $h[$d + 1][$d + 1] - $z - $r - $s;
+				$r = $h[$d + 2][$d + 1];
 
 				$s = abs($p) + abs($q) + abs($r);
 				$p /= $s;
 				$q /= $s;
 				$r /= $s;
 
-				last if ($m == $l);
-				last if (
-					abs($h[$m][$m - 1]) * (abs($q) + abs($r)) <=
+				#
+				# The sub-diagonal check doesn't get made for
+				# the last iteration of the loop, and the only
+				# reason we have the loop continue up to this
+				# point is to set $p, $q, and $r.
+				#
+				last if ($d == $l);
+
+				if (abs($h[$d][$d - 1]) * (abs($q) + abs($r)) <=
 					$epsilon * abs($p) * (
-						  abs($h[$m - 1][$m - 1]) +
+						  abs($h[$d - 1][$d - 1]) +
 						  abs($z) +
-						  abs($h[$m + 1][$m + 1])
-					)
-				  );
+						  abs($h[$d + 1][$d + 1])
+					))
+				{
+					$m = $d;
+					last;
+				}
 			}
 
+			#
+			#### $n
+			#### $l
+			#### $m
+			#
 			for my $i (($m + 2) .. $n)
 			{
 				$h[$i][$i - 2] = 0.0;
@@ -856,6 +888,7 @@ sub hqr_eigen_hessenberg
 			#
 			for my $k ($m .. $na)
 			{
+				my $z;
 				my $notlast = ($k != $na);
 				if ($k != $m)
 				{
@@ -871,7 +904,7 @@ sub hqr_eigen_hessenberg
 					$r /= $x;
 				}
 
-				$s = sqrt($p * $p + $q * $q + $r * $r);
+				my $s = sqrt($p * $p + $q * $q + $r * $r);
 				$s = -$s if ($p < 0.0);
 
 				if ($k != $m)
@@ -915,7 +948,8 @@ sub hqr_eigen_hessenberg
 				#
 				for my $i ($l .. $j)
 				{
-					$p = $x * $h[$i][$k] + $y * $h[$i][$k + 1];
+					$p = $x * $h[$i][$k] +
+					     $y * $h[$i][$k + 1];
 
 					if ($notlast)
 					{
@@ -1002,9 +1036,10 @@ sub poly_roots
 	{
 		my $matrix_ref = build_companion(@coefficients);
 
-		#### Balanced Companion Matrix (row and column 0 will be undefs)...
-		#### $matrix_ref
-
+		#
+		### Balanced Companion Matrix
+		##### (row and column 0 will be undefs)...
+		##### $matrix_ref
 		#
 		# QR iterations from the matrix.
 		#
@@ -1083,7 +1118,7 @@ sub poly_analysis
 		#
 		map(($m *= $_), @czp);
 
-		### Substitution degree: $m
+		#### Substitution degree: $m
 	}
 
 	#
@@ -1209,7 +1244,7 @@ sub poly_evaluate
 		#
 		# It could happen. Someone might type \$x instead of $x.
 		#
-		@values = ( (ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
+		@values = ((ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
 	}
 
 	#
@@ -1229,6 +1264,65 @@ sub poly_evaluate
 	return wantarray? @results: $results[0];
 }
 
+#
+# ($y, $dy, $ddy) = poly_derivaluate(\@coefficients, $x);
+#
+# Returns p(x), p'(x), and p"(x) of the polynomial for an
+# x-value, using Horner's method.
+#
+sub poly_derivaluate
+{
+	my($coef_ref, $x) = @_;
+	my(@coefficients) = @$coef_ref;
+	my $n = $#coefficients;
+	my $val = shift @coefficients;
+	my $d1val = $val * $n;
+	my $d2val = 0;
+
+	#
+	# Be nice and check if the user accidentally passed in
+	# a reference for the $x value.
+	### poly_derivaluate
+	#### $coef_ref
+	#### $x
+	#
+	croak "Used a reference instead of an X value in poly_derivaluate()" if (ref $x eq "ARRAY" or ref $x eq "SCALAR");
+
+	#
+	# Special case for the linear eq'n (the y = constant eq'n
+	# takes care of itself).
+	#
+	if ($n == 1)
+	{
+		$d1val = $val;
+		$val = $val * $x + $coefficients[0];
+	}
+	elsif ($n >= 2)
+	{
+		my $lastn = --$n;
+		$d2val = $d1val * $n;
+
+		#
+		# Loop through the coefficients, except for
+		# the linear and constant terms.
+		#
+		foreach my $c (@coefficients[0..$lastn-2])
+		{
+			$val = $val * $x + $c;
+			$d1val = $d1val * $x + ($c *= $n--);
+			$d2val = $d2val * $x + ($c * $n);
+		}
+
+		#
+		# Handle the last two coefficients.
+		#
+		$d1val = $d1val * $x + $coefficients[$lastn-1];
+		$val = ($val * $x + $coefficients[$lastn-1]) * $x + $coefficients[$lastn];
+	}
+
+	return ($val, $d1val, $d2val);
+}
+
 #
 # ($q_ref, $r_ref) = poly_divide(\@coefficients1, \@coefficients2);
 #
@@ -1258,10 +1352,10 @@ sub poly_divide
 	return (undef, undef) if ($d_degree < 0);
 
 	#
-	###Dividing:
-	### @numerator
-	### by
-	### @divisor
+	### poly_divide():
+	#### @numerator
+	#### by
+	#### @divisor
 	#
 	my $lead_coefficient = $divisor[0];
 
@@ -1337,7 +1431,7 @@ sub poly_sturm_chain
 	while ($#$r > 0);
 
 	#
-	#### poly_sturm_chain:
+	### poly_sturm_chain:
 	#### @chain
 	#
 	return @chain;
@@ -1364,8 +1458,8 @@ sub poly_real_root_count
 # $root_count = sturm_real_root_range_count(\@chain, $x0, $x1);
 #
 # An all-in-one function for finding the number of real roots in
-# a polynomial. Use this if you don't intend to do anything else
-# requiring the Sturm chain.
+# a polynomial over a range in X. Use this if you don't intend to do
+# anything else requiring the Sturm chain.
 #
 sub sturm_real_root_range_count
 {
@@ -1428,12 +1522,12 @@ sub sturm_bisection_roots
 			my $midto_count = sturm_real_root_range_count($chain_ref, $mid, $to);
 
 			#
-			### $its
-			### $from
-			### $to
-			### $mid
-			### $frommid_count
-			### $midto_count
+			#### $its
+			#### $from
+			#### $to
+			#### $mid
+			#### $frommid_count
+			#### $midto_count
 			#
 
 			#
@@ -1571,14 +1665,17 @@ sub sturm_sign_count
 	return $scnt;
 }
 
+#
+# @roots = laguerre(\@coefficients, \@xvals);
+#
+# Find the roots nearby the given X values.
+#
 sub laguerre
 {
+	no Math::Complex;
 	my($p_ref, $xval_ref) = @_;
-	my(@p0) = @$p_ref;
-	my(@p1) = poly_derivative(@p0);
-	my(@p2) = poly_derivative(@p1);
-	my $n = $#p0;
-	my @values;
+	my $n = $#$p_ref;
+	my @xvalues;
 	my @roots;
 
 	#
@@ -1586,49 +1683,56 @@ sub laguerre
 	#
 	if (ref $xval_ref eq "ARRAY")
 	{
-		@values = @$xval_ref;
+		@xvalues = @$xval_ref;
 	}
 	else
 	{
 		#
 		# It could happen. Someone might type \$x instead of $x.
 		#
-		@values = ( (ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
+		@xvalues = ((ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
 	}
 
-	foreach my $x (@values)
+	foreach my $x (@xvalues)
 	{
+		#
+		### laguerre: looking near $x
+		### Coefficient: @$p_ref
+		### Degree: $n
+		#
 		my $its = 0;
 
 		ROOT:
 		for (;;)
 		{
 			#
-			# There's a better way to do this, but
-			# I'm going for "simple to code" for
-			# this developer release.
+			# Get the values of the function and its first and
+			# second derivatives at X.
 			#
-			my $valp0 = poly_evaluate($p_ref, $x);
-			my $valp1 = poly_evaluate(\@p1, $x);
-			my $valp2 = poly_evaluate(\@p2, $x);
+			my($y, $dy, $d2y) = poly_derivaluate($p_ref, $x);
 
-			if (abs($valp0) <= $tolerance{laguerre})
+			if (abs($y) <= $tolerance{laguerre})
 			{
 				push @roots, $x;
 				last ROOT;
 			}
 
 			#
-			### At Iteration: $its
-			### X: $x
-			### f(x): $valp0
-			### f'(x): $valp1
-			### f''(x): $valp2
+			#### At Iteration: $its
+			#### X: $x
+			#### f(x): $y
+			#### f'(x): $dy
+			#### f''(x): $d2y
 			#
-			my $g = $valp1/$valp0;
-			my $h = $g * $g - $valp2/$valp0;
+			my $g = $dy/$y;
+			my $h = $g * $g - $d2y/$y;
 			my $f = sqrt(($n - 1) * ($n * $h - $g*$g));
+			$f = - $f if (abs($g - $f) > abs($g + $f));
 
+			#
+			#### g = $g
+			#### h = $h
+			#### f = $f
 			#
 			# Divide by the largest value of $g plus
 			# $f, bearing in mind that $f is the result
@@ -1638,7 +1742,6 @@ sub laguerre
 			# Use the abs() function to determine size
 			# since $g or $f may be complex numbers.
 			#
-			$f = - $f if (abs($g - $f) > abs($g + $f));
 			my $dx = $n/($g + $f);
 
 			$x -= $dx;
@@ -1651,6 +1754,79 @@ sub laguerre
 			croak "Too many iterations ($its) at dx=$dx\n" if ($its >= $iteration{laguerre});
 			$its++;
 		}
+
+		### root found at iteration $its
+		#### $x
+	}
+	return @roots;
+}
+
+#
+# @xvals = newtonraphson(\@coefficients, \@xvals);
+#
+# Find the roots nearby the given X values.
+#
+sub newtonraphson
+{
+	no Math::Complex;
+	my($p_ref, $xval_ref) = @_;
+	my $n = $#$p_ref;
+	my @xvalues;
+	my @roots;
+
+	#
+	# Allow some flexibility in sending the x-values.
+	#
+	if (ref $xval_ref eq "ARRAY")
+	{
+		@xvalues = @$xval_ref;
+	}
+	else
+	{
+		#
+		# It could happen. Someone might type \$x instead of $x.
+		#
+		@xvalues = ((ref $xval_ref eq "SCALAR")? $$xval_ref: $xval_ref);
+	}
+
+	#
+	### newtonraphson()
+	#### @xvalues
+	#
+	foreach my $x (@xvalues)
+	{
+		my $its = 0;
+
+		ROOT:
+		for (;;)
+		{
+			#
+			# Get the values of the function and its first and
+			# second derivatives at X.
+			#
+			my($y, $dy, $d2y) = poly_derivaluate($p_ref, $x);
+			my $dx = $y/$dy;
+			$x -= $dx;
+
+			if (abs($dx) <= $tolerance{newtonraphson})
+			{
+				push @roots, $x;
+				last ROOT;
+			}
+
+			#
+			#### At Iteration: $its
+			#### X: $x
+			#### f(x): $y
+			#### f'(x): $dy
+			#### f''(x): $d2y
+			#
+			croak "Too many iterations ($its) at dx=$dx\n" if ($its >= $iteration{newtonraphson});
+			$its++;
+		}
+
+		### root found at iteration $its
+		#### $x
 	}
 	return @roots;
 }
@@ -1664,6 +1840,8 @@ sub poly_gcd
 1;
 __END__
 
+=encoding utf8
+
 =head1 NAME
 
 Math::Polynomial::Solve - Find the roots of polynomial equations.
@@ -1739,6 +1917,8 @@ or
 This package supplies a set of functions that find the roots of
 polynomials, along with some utility functions.
 
+Roots will be either real or of type L<Math::Complex>.
+
 Functions making use of the Sturm sequence are also available, letting you
 find the number of real roots present in a range of X values.
 
@@ -1782,9 +1962,9 @@ B<NOTE>: this function is replaced by the option function C<poly_option()>.
 
 =head3 poly_option()
 
-Set options that affect the behavior of the C<poly_roots()> function. All options
-are set to either 1 ("on") or 0 ("off"). See also L</poly_iteration()>
-and L/<poly_tolerance()>.
+Set options that affect the behavior of the C<poly_roots()> function. All
+options are set to either 1 ("on") or 0 ("off"). See also L</poly_iteration()>
+and L</poly_tolerance()>.
 
 This is the option function that deprecates C<set_hessenberg()> and
 C<get_hessenberg()>.
@@ -1828,16 +2008,18 @@ solving methods.
 
 Reduce polynomials to a lower degree through variable substitution, if possible.
 
-For example, with varsubst set to one and the polynomial to solve is C<9x**6 + 128x**3 + 21>,
-then C<poly_roots()> will reduce the polynomial to C<9y**2 + 128y + 21>, where y = x**3,
+For example, with C<varsubst> set to one and the polynomial to solve being
+C<9x**6 + 128x**3 + 21>, C<poly_roots()> will reduce the polynomial to
+C<9y**2 + 128y + 21> (where C<y = x**3>),
 and solve the quadratic (either classically or numerically, depending
 on the hessenberg option). Taking the cube root of each quadratic root
 completes the operation.
 
-This has the benefit of having a simpler matrix to solve, or if the hessenberg option
-is set to zero, has the effect of being able to use one of the classical methods on a
-polynomial of high degree. In the above example, the order-six polynomial gets solved
-with the quadratic_roots() function if the hessenberg option is zero.
+This has the benefit of having a simpler matrix to solve, or if the
+C<hessenberg> option is set to zero, has the effect of being able to use one of
+the classical methods on a polynomial of high degree. In the above example, the
+order-six polynomial gets solved with the quadratic_roots() function if the
+hessenberg option is zero.
 
 Currently the variable substitution is fairly simple and will only look
 for gaps of zeros in the coefficients that are multiples of the prime numbers
@@ -1907,8 +2089,9 @@ Return the number of I<unique>, I<real> roots of the polynomial.
 
   $unique_roots = poly_real_root_count(@coefficients);
 
-For example, the equation C<(x + 3)**3> forms the polynomial C<x**3 + 9x**2 + 27x + 27>,
-but since all three of its roots are identical, C<poly_real_root_count(1, 9, 27, 27)> will return 1.
+For example, the equation C<(x + 3)**3> forms the polynomial
+C<x**3 + 9x**2 + 27x + 27>, but since all three of its roots are identical,
+C<poly_real_root_count(1, 9, 27, 27)> will return 1.
 
 Likewise, C<poly_real_root_count(1, -8, 25)> will return 0 because the two roots
 of C<x**2 -8x + 25> are both complex.
@@ -1922,6 +2105,37 @@ This function is the all-in-one function to use instead of
 
 if you don't intend to use the Sturm chain for anything else.
 
+=head3 sturm_real_root_range_count()
+
+Return the number of I<unique>, I<real> roots of the polynomial between two X values.
+
+  my($x0, $x1) = (0, 1000);
+
+  my @chain = poly_sturm_chain(@coefficients);
+  $unique_roots = sturm_real_root_range_count(\@chain, $x0, $x1);
+
+This is equivalent to:
+
+  my($x0, $x1) = (0, 1000);
+
+  my @chain = poly_sturm_chain(@coefficients);
+  my @signs = sturm_sign_chain(\@chain, [$x0, $x1]);
+  $unique_roots = sturm_sign_count(@{$signs[0]}) - sturm_sign_count(@{$signs[1]});
+
+=head3 sturm_bisection_roots()
+
+Return the I<real> roots counted by L</sturm_real_root_range_count()>. Uses the
+bisection method combined with C<sturm_real_range_count()> to narrow the range
+to a single root, then uses L</laguerre()> to find the value.
+
+  my($from, $to) = (-1000, 0);
+  my @chain = poly_sturm_chain(@coefficients);
+  my @roots = sturm_bisection_roots(\@chain, $from, $to);
+
+As it is using the Sturm functions, it will find only the real roots.
+
+Internally, laguerre() is used by sturm_bisection_roots().
+
 =head3 poly_sturm_chain()
 
 Returns the chain of Sturm functions used to evaluate the number of roots of a
@@ -1933,20 +2147,6 @@ times the Y values change sign.
 
 See L</poly_real_root_count()> above for an example of its use.
 
-=head3 sturm_real_root_range_count()
-
-Return the number of I<unique>, I<real> roots of the polynomial.
-
-  my($x0, $x1) = (0, 1000);
-
-  my @chain = poly_sturm_chain(@coefficients);
-  $unique_roots = sturm_real_root_range_count(\@chain, $x0, $x1);
-
-Internally, this is equivalent to:
-
-  my @signs = sturm_sign_chain(\@chain, [$x0, $x1]);
-  return sturm_sign_count(@{$signs[0]}) - sturm_sign_count(@{$signs[1]});
-
 =head3 Sturm Sign Sequence Functions
 
 =head4 sturm_sign_chain()
@@ -2012,26 +2212,16 @@ such as those returned by the L</Sturm Sign Sequence Functions>
 See L</poly_real_root_count()> and L</sturm_real_root_range_count()> for
 examples of its use.
 
-=head3 sturm_bisection_roots()
-
-Return the I<real> roots counted by L</sturm_real_root_range_count()>. Uses the
-bisection method combined with C<sturm_real_range_count()> to narrow the range
-to a single root, then uses L</laguerre()> to find the value.
-
-  my($from, $to) = (-1000, 0);
-  my @chain = poly_sturm_chain(@coefficients);
-  my @roots = sturm_bisection_roots(\@chain, $from, $to);
-
-As it is using the Sturm functions, it will find only the real roots.
-
 =head2 Utility Functions
 
-These are internal functions used by the other functions listed above,
-but which may also be useful to the user. They are all exported under the tag "utility".
+These are internal functions used by the other functions listed above
+that may also be useful to the user, or which affect the behavior of
+other functions. They are all exported under the tag "utility".
 
 =head3 epsilon()
 
-Returns the machine epsilon value that was calculated when this module was loaded.
+Returns the machine epsilon value that was calculated when this module was
+loaded.
 
 The value may be changed, although this in general is not recommended.
 
@@ -2078,14 +2268,31 @@ for polynomials.
   push @roots, laguerre(\@coefficients, $another_xvalue);
 
 For each x value the function will attempt to find a root closest to it.
+The function will return real roots only.
+
+This is the function used by L</sturm_bisection_roots()> after narrowing its
+search to a range containing a single root.
+
+=head3 newtonraphson()
+
+Like L</laguerre()>, a numerical method for finding a root of an equation.
+
+  @roots = newtonraphson(\@coefficients, \@xvalues);
+  push @roots, newtonraphson(\@coefficients, $another_xvalue);
+
+For each x value the function will attempt to find a root closest to it.
+The function will return real roots only.
+
+This function is provided as an alternative to laguerre(). It is not
+used internally by any other functions.
 
 =head3 poly_iteration()
 
 Sets the limit to the number of iterations that a solving method may go
 through before giving up trying to find a root. Each method of root-finding
 used by L</poly_roots()>, L</sturm_bisection_roots()>, and L</laguerre()>
-has its own iteration limit, which may be found, like L</poly_option()>, by
-simply looking at the return value of poly_iteration().
+has its own iteration limit, which may be found, like L</poly_option()>,
+simply by looking at the return value of poly_iteration().
 
   #
   # Get all of the current iteration limits.
@@ -2096,13 +2303,14 @@ simply looking at the return value of poly_iteration().
   # Double the limit for the hessenberg method, but set the limit
   # for Laguerre's method to 20.
   #
-  poly_iteration(hessenberg => $its_limits{hessenberg} * 2, laguerre => 12);
+  my %old_limits = poly_iteration(hessenberg => $its_limits{hessenberg} * 2,
+                      laguerre => 20);
 
   #
   # Reset the limits with the former values, but save the values we had
   # for later.
   #
-  my %hl_limits = poly_iteration(%its_limits);
+  my %hl_limits = poly_iteration(%old_limits);
 
 There are iteration limit values for:
 
@@ -2115,10 +2323,15 @@ Its default value is 60.
 
 =item laguerre
 
-The numeric method used by laguerre(). Laguerre's method is used within
-sturm_bisection_roots() once an individual root has been found within a
-range, and of course it may be called independently. Its default value is
-60.
+The numeric method used by L</laguerre()>. Laguerre's method is used within
+sturm_bisection_roots() once it has narrowed its search in on an individual
+root, and of course laguerre() may be called independently. Its default value
+is 60.
+
+=item newtonraphson
+
+The numeric method used by newtonraphson(). The Newton-Raphson method is offered
+as an alternative to Laguerre's method.  Its default value is 60.
 
 =item sturm_bisection
 
@@ -2131,8 +2344,8 @@ is 100.
 
 Set the degree of accuracy needed for comparisons to be equal or roots to
 be found.  Amongst the root finding functions this currently only
-affects laguerre(), as the Hessenberg matrix method determines how close
-it needs to get using a complicated formula based on L</epsilon()>.
+affects laguerre() and newtonraphson(), as the Hessenberg matrix method determines
+how close it needs to get using a complicated formula based on L</epsilon()>.
 
   #
   # Print the tolerances.
@@ -2159,6 +2372,11 @@ The numeric method used by laguerre(). Laguerre's method is used within
 sturm_bisection_roots() once an individual root has been found within a range,
 and of course it may be called independently.
 
+=item newtonraphson
+
+The numeric method used by newtonraphson(). Newton-Raphson is, like Laguerre's
+method, a method for finding a root near the starting X value.
+
 =item fltcmp
 
 A comparison function that determines if one argument is less than, equal to,
@@ -2205,7 +2423,8 @@ The function may return a list...
 
   print "Polynomial: [", join(", ", @coefficients), "]\n";
 
-  for my $j (0..$#yvals) {
+  for my $j (0..$#yvals)
+  {
     print "Evaluates at ", $xvals[$j], " to ", $yvals[$j], "\n";
   }
 
@@ -2215,6 +2434,20 @@ or return a scalar.
   my $y_median = poly_evaluate(\@coefficients, $x_median);
 
 
+=head3 poly_derivaluate();
+
+Given an X value, returns the y-values of the polynomial, its first derivative,
+and its second derivative.
+
+  my($y, $dy, $ddy) = poly_derivaluate(\@coefficients, $x);
+
+Note that unlike L</poly_evaluate()>, this takes a single
+x-value.
+
+If the polynomial is a linear equation, the second derivative value will be
+zero.  Similarly, if the "equation" is a constant, the first derivative value
+will be zero.
+
 =head3 poly_nonzero_term_count()
 
 Returns a simple count of the number of coefficients that aren't zero.
@@ -2315,7 +2548,8 @@ Eigenvalues", Math. Comp., v64,#210, pp.763-776(1995).
 
 For starting out, you may want to read
 
-William Press, Brian P. Flannery, Saul A. Teukolsky, and William T. Vetterling I<Numerical Recipes in C>.
+William Press, Brian P. Flannery, Saul A. Teukolsky, and William T. Vetterling
+I<Numerical Recipes in C>.
 Cambridge University Press, 1988.
 They have a web site for their book, L<http://www.nr.com/>.
 
@@ -2346,10 +2580,17 @@ Acton, Forman S. I<Numerical Methods That Work>. New York: Harper & Row, Publish
 =over 5
 
 Lively, opinionated book on numerical equation solving. I looked it up when it
-became obvious that everyone was quoting Acton when discussing Laguerre's method.
+became obvious that everyone was quoting Acton when discussing Laguerre's
+method.
 
 =back
 
+=head2 Newton-Raphson
+
+Commonly known as Newton's method. Almost every introduction to calculus
+text book will have a section on it; a Wikipedia article is at
+L<http://en.wikipedia.org/wiki/Newton%27s_method>.
+
 =head1 SEE ALSO
 
 Forsythe, George E., Michael A. Malcolm, and Cleve B. Moler
